package SeMeet::Controller::Users;
use Mojo::Base 'Mojolicious::Controller', -signatures;

use utf8;
use JSON;
use constant REDIS_CACHE_LIFETIME => 60;

sub list($c) {
    $c->openapi->valid_input or return;
    my $args = $c->validation->output;

    my $cache_key = sprintf('USERS %s', $args->{search} || 'ALL');
    if ($c->redis->db->exists( $cache_key )) {
        $c->render(openapi => from_json($c->redis->db->get( $cache_key )));
        return;
    }

    my $groups = $c->schema->resultset('Group')->search(
        { permissions => { ilike => 'org-%' }}, # TODO: samostatny priznak?
        { order_by => 'name', }
    );

    my %groups = ();

    GROUP:
    while ( my $group = $groups->next()) {
        my $name = $group->name;

        next GROUP if $name !~ s/^(KS|MS)\s+//i;

        my $type = $1;

        $name =~ s/\s+členové a RegP//i;
        $groups{$type}->{$group->octid} = $name;
    }

    my $users = $c->iapi->get('octopus/users?search=' . $args->{search});

    my @users = ();

    USER:
    foreach my $user ( @{ $users } ) {

        my ($ks, $ms);

        GROUP:
        foreach my $id (@{ $user->{groups} }) {
            if ( my $group = $groups{KS}->{$id} ) {
                $ks //= $group;
            }
            if ( my $group = $groups{MS}->{$id} ) {
                $ms //= $group;
            }
        }

        push @users, $c->spec_filter({
            id          => $user->{id},
            name        => $user->{displayname},
            username    => $user->{username},
            region      => join ' - ', grep { defined } ($ks, $ms),
        }, 'UserInList')
    }

    $c->redis->db->set( $cache_key, to_json(\@users), 'EX', REDIS_CACHE_LIFETIME );
    $c->render( openapi => \@users );
}

1;

